/*
 * RT2X00 Compatability fixes for specific kernels.
 */
#ifndef RT2X00_COMPAT_H
#define RT2X00_COMPAT_H

/*
 * First include the 2 config headers.
 * The rt2x00_config.h should overrule
 * the kernel configuration.
 */
#include <linux/config.h>

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>

/*
 * Check minimal requirements.
 */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 17)
#error Invalid kernel version!
#error Please refer to README file for minimal requirements.
#endif

#if !defined(CONFIG_NET_RADIO)
#error Wireless radio support not enabled in kernel!
#error Please refer to README file for minimal requirements.
#endif

#if (!defined(CONFIG_CRYPTO) && !defined(CONFIG_CRYPTO_MODULE))
#error Crypto support not enabled in kernel!
#error Please refer to README file for minimal requirements.
#endif

#if (!defined(CONFIG_CRYPTO_AES) && !defined(CONFIG_CRYPTO_AES_MODULE))
#if (!defined(CONFIG_CRYPTO_AES_586) && !defined(CONFIG_CRYPTO_AES_586_MODULE))
#if (!defined(CONFIG_CRYPTO_AES_X86_64) && !defined(CONFIG_CRYPTO_AES_X86_64_MODULE))
#error Crypto AES support not enabled in kernel!
#error Please refer to README file for minimal requirements.
#endif
#endif
#endif

#if (!defined(CONFIG_CRYPTO_ARC4) && !defined(CONFIG_CRYPTO_ARC4_MODULE))
#error Crypto ARC4 support not enabled in kernel!
#error Please refer to README file for minimal requirements.
#endif

#if (!defined(CONFIG_D80211) && !defined(CONFIG_D80211_MODULE))
#error d80211 support not enabled in kernel, nor is it build as external module!
#error Please refer to README file for minimal requirements.
#endif

#if (defined(CONFIG_RT2400PCI) || defined(CONFIG_RT2500PCI) || defined(CONFIG_RT61PCI))
#if (!defined(CONFIG_PCI) && !defined(CONFIG_PCI_MODULE))
#error PCI has been disabled in your kernel!
#error Please refer to README file for minimal requirements.
#endif
#endif

#if (defined(CONFIG_RT2500USB) || defined(CONFIG_RT73USB))
#if (!defined(CONFIG_USB) && !defined(CONFIG_USB_MODULE))
#error USB has been disabled in your kernel!
#error Please refer to README file for minimal requirements.
#endif
#endif

#if (defined(CONFIG_RT61PCI) || defined(CONFIG_RT73USB))
#if (!defined(CONFIG_FW_LOADER) && !defined(CONFIG_FW_LOADER_MODULE))
#error Firmware loading has been disabled in your kernel!
#error Please refer to README file for minimal requirements.
#endif
#if (!defined(CONFIG_CRC_ITU_T) && !defined(CONFIG_CRC_ITU_T_MODULE))
#error CRC_ITU_T loading has been disabled in your kernel!
#error Please refer to README file for minimal requirements.
#endif
#endif

#if (defined(CONFIG_RFKILL) || defined (CONFIG_RFKILL_MODULE))
#if (!defined(CONFIG_INPUT) && !defined(CONFIG_INPUT_MODULE))
#error Input device support has been disabled in your kernel!
#error Please refer to README file for minimal requirements.
#endif
#endif

#if (defined(CONFIG_RT2400PCI_BUTTON) || defined(CONFIG_RT2500PCI_BUTTON) || defined(CONFIG_RT61PCI_BUTTON))
#if (!defined(CONFIG_RFKILL) && !defined (CONFIG_RFKILL_MODULE))
#error Attemtping to build hardware button support without rfkill driver!
#error Please refer to README file for minimal requirements.
#endif
#endif

/*
 * Backward compatibility fixes for all kernels starting from 2.6.17
 */
#if LINUX_VERSION_CODE == KERNEL_VERSION(2, 6, 17)
/*
 * From 2.6.18 and above all access to the xmit_lock should
 * go through the netif_tx_lock_bh and netif_tx_unlock_bh functions.
 */
#define netif_tx_lock_bh(__dev)					\
	do {							\
		spin_lock_bh(&(__dev)->xmit_lock);		\
		(__dev)->xmit_lock_owner = smp_processor_id();	\
	} while (0)

#define netif_tx_unlock_bh(__dev)			\
	do {						\
		(__dev)->xmit_lock_owner = -1;		\
		spin_unlock_bh(&(__dev)->xmit_lock);	\
	} while (0)

/*
 * SA_SHIRQ has been deprecated in kernel 2.6.18
 */
#define IRQF_SHARED	SA_SHIRQ

#endif /* LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 17) */

/*
 * The net_device ieee80211_ptr field is not available in any kernel
 * except for the dscape and wireless-dev git trees.
 * We are going to use the net_device base_addr field instead.
 */
#define get_80211_ptr(__dev)		( (void*)(__dev)->base_addr )
#define set_80211_ptr(__dev, __ptr)	( (__dev)->base_addr = (unsigned long)(__ptr) )

/*
 * For rfkill driver, this driver is not yet in the kernel,
 * and neither is this KEY define which should go into linux/input.h
 */
#define KEY_RFKILL		0x1a0

#endif /* RT2X00_COMPAT_H */
